@@ -78,6 +78,15 @@ var UI = {
       return 0;
     }
   },
+  adjust_exam_sector:function(data){
+    let l=data.byteLength;
+    if (l<0x3000) return;
+    let tab=new Uint8Array(data);
+    for (let i=0x1000;i<0x2000;i++)
+      tab[i]=0xff;
+    for (let i=0x2000;i<0x3000;i++)
+      tab[i]=0x0;
+  },
   sig_check:async function(sig,data,fname){
     // sig should be a list of lists of size 3 (name, length, hash)
     let i=0,l=sig.length;
@@ -198,20 +207,26 @@ var UI = {
   loadFile:function(file){
     let reader = new FileReader();
     reader.onerror = function (evt) { }
-    if (file.name.length>4 && file.name.substr(file.name.length-4,4)==".nws"){
+    let backup=file.name.length>4 && file.name.substr(file.name.length-4,4)==".nws";
+    if (backup){
       if (!confirm(UI.langue==-1?'Remplacer tous les scripts de la calculatrice?':'Overwrite all calculator scripts?'))
 	return;
       reader.readAsArrayBuffer(file);
     }
     else
       reader.readAsText(file, "UTF-8");
-    reader.onload = function (evt) { UI.numworks_save(file.name,evt.target.result);}
+    reader.onload = function (evt) {
+      let data=evt.target.result;
+      if (!backup)
+	data+=String.fromCharCode(0);
+      UI.numworks_save(file.name,data);
+    }
   },
   numworks_save:function(filename,S){
     //console.log(filename,S); return;
     UI.nws_connect();
     //window.setTimeout(UI.numworks_save_,100,filename,S);
-    window.setTimeout(UI.numworks_save_,100,filename,S+String.fromCharCode(0));
+    window.setTimeout(UI.numworks_save_,100,filename,S);
   },
   numworks_save_:async function(filename,S){
     if (UI.calculator==0 || !UI.calculator_connected){
@@ -222,8 +237,10 @@ var UI = {
     if (filename.length>4 && filename.substr(filename.length-4,4)==".nws"){
       let pinfo = await UI.calculator.getPlatformInfo();
       UI.calculator.device.startAddress = pinfo["storage"]["address"];
-      // console.log(UI.calculator.device.startAddress,S); // return;
+      //console.log(UI.calculator.device.startAddress,S); return;
       let res=await UI.calculator.device.do_download(UI.calculator.transferSize, S, false);
+      if (res==0)
+	alert('Unable to store to the calculator');
       return res;
     }
     if (UI.is_py(filename))
@@ -257,33 +274,98 @@ var UI = {
     window.setTimeout(UI.numworks_install_delta_,100,do_backup,khionly);
   },
   numworks_install_delta_:async function(do_backup,khionly){
+    if (khionly==5){
+      UI.calculator.device.startAddress = 0x90180000;
+      UI.calculator.device.logProgress(0,'khi slot B');
+      let data=await UI.loadfile('khi.B.bin');
+      let res=await UI.calculator.device.do_download(UI.calculator.transferSize, data, false);
+      if (res==0){
+	alert(UI.langue==-1?'Erreur d\'ecriture flash. Tapez reset+4 et reessayez':'Error writing flash. Type reset+4 and try again.');
+	return 3;
+      }
+      alert(UI.langue==-1?'Tapez reset+2 pour booter un lanceur minimal de KhiCAS sur le slot B. Ceci augmente la memoire de 50% environ. Attention, cela ne fonctionnera que si vous installez le bootloader de cette page!':'Type reset+2 to boot a minimal KhiCAS launcher from slot B. This increases memory available. Beware this works only if you install the bootloader of this page!');
+      UI.print('Khi slot B OK, press RESET on calculator back');
+      return 0;
+    }
+    if (khionly==4){
+      UI.calculator.device.startAddress = 0x08000000;
+      UI.calculator.device.logProgress(0,'protection');
+      let data=await UI.loadfile('bootloader.bin');
+      let res=0;
+      res=await UI.calculator.device.do_download(UI.calculator.transferSize, data, false);
+      if (res==0){
+	alert(UI.langue==-1?'Erreur d\'ecriture flash interne. Tapez reset+6 et reessayez. Si reset+6 ne fonctionne pas, essayez de deverrouiller votre Numworks sur phi.getomega.dev':'Error writing internal flash. Type reset+6 and try again. If reset+6 does not work, try to unlock your Numworks on phi.getomega.dev');
+	return 3;
+      }
+      alert(UI.lang==-1?'Le multi-boot permet de lancer 2 firmwares A (<=1.5M) et B(<=0.5M) : reset-1 ou reset-2 permet de changer. Si vous installez Khi A et B, votre calculatrice sera protegee contre des mises a jour (risque de verrouillage). Cette protection se desactive temporairement par reset-4, par exemple pour mettre a jour Khi. Tapez reset+6 pour effacer le multiboot':'With multi-boot installed you can run 2 firmwares A (<=1.5M) and B (<=0.5M). If you install Khi A and B, your calculator will be protected against upgrade (lock risk). Type reset-4 to temporarily remove firmware protection, e.g. to upgrade Khi. Type reset-6 to remove the multi-boot');
+      UI.print('Bootloader OK, press RESET on calculator back');
+      return 0;
+    }
     if (UI.calculator==0 || !UI.calculator_connected){
       alert(UI.langue==-1?UI.chkmsgfr:chkmsgen);
       if (UI.calculator) UI.calculator.stopAutoConnect();
       return -1;
     } 
     UI.print(khionly?'=========== installing Khi':'=========== installing Khi and KhiCAS');
+    let data,res;
+    if (!khionly){
+      UI.print('erase/write KhiCAS apps, wait about 2 minutes');
+      data=await UI.loadfile('apps.tar');
+      UI.calculator.device.startAddress = 0x90200000;
+      UI.calculator.device.logProgress(0,'apps');
+      res=await UI.calculator.device.do_download(UI.calculator.transferSize, data, false);
+      UI.print(res?'KhiCAS installed':'error installing KhiCAS');
+      if (res==0){
+	alert('error installing KhiCAS');
+	return 1;
+      }
+    }
+    let installboot=0;
+    let pinfo = await UI.calculator.getPlatformInfo();
+    if (pinfo["bootloader"]==0 || khionly==3){
+      installboot=1;
+      if (khionly==3) // skip protection question parameter
+	khionly=1;
+      else {
+	if (1 ||
+	    confirm(UI.langue==-1?'Votre calculatrice n\'est pas protegee contre un verrouillage. Voulez-vous la proteger?':'Your calculator is not protected against an hostile lock. Protect?')){
+	  alert(UI.langue==-1?'Le bootloader de protection sera installe. Vous pourrez l\'effacer avec reset+6. Attention, ceci ne protege pas d\'une mise a jour sur le site de Numworks.':'Protecting bootloader will be installed, you will be able to remove it with reset+6. Beware, this does not protect from running an upgrade from Numworks.');
+	  khionly=2;
+	}
+	else
+	  khionly=1;
+      }
+    }
+    else
+      khionly=2;
     UI.print('erase/write Khi external, wait about 1/2 minute');
     UI.calculator.device.startAddress = 0x90000000;
-    let data=await UI.loadfile('delta.external.bin');
-    UI.calculator.device.logProgress(0,'external');
-    let res=await UI.calculator.device.do_download(UI.calculator.transferSize, data, false);
-    UI.print('Khi external OK, erase/write internal');
-    UI.calculator.device.startAddress = 0x08000000;
-    UI.calculator.device.logProgress(0,'internal');
-    data=await UI.loadfile('delta.internal.bin');
+    let firmwarename=khionly==2?'khi.A.bin':'delta.external.bin';
+    data=await UI.loadfile(firmwarename);
+    UI.calculator.device.logProgress(0,firmwarename);
     res=await UI.calculator.device.do_download(UI.calculator.transferSize, data, false);
-    if (khionly){
-      UI.print('Khi internal OK, press RESET on calculator back');
-      alert(UI.langue==-1?'Installation terminee. Appuyer sur RESET a l\'arriere de la calculatrice':'Install success. Press RESET on the calculator back.');
-      return;
+    if (res==0){
+      alert('Error writing Khi');
+      return 2;
     }
-    UI.print('Khi internal OK, erase/write KhiCAS and apps, wait about 2 minutes');
-    UI.calculator.device.startAddress = 0x90200000;
-    UI.calculator.device.logProgress(0,'apps');
-    data=await UI.loadfile('apps.tar');
+    UI.calculator.device.startAddress = 0x90180000;
+    UI.calculator.device.logProgress(0,'khi slot 2');
+    data=await UI.loadfile('khi.B.bin');
     res=await UI.calculator.device.do_download(UI.calculator.transferSize, data, false);
-    UI.print('apps OK, press RESET on calculator back');
+    if (installboot){
+      UI.print('Khi external OK, erase/write internal');
+      UI.calculator.device.startAddress = 0x08000000;
+      UI.calculator.device.logProgress(0,'internal');
+      data=await UI.loadfile(khionly==2?'bootloader.bin':'delta.internal.bin');
+      res=await UI.calculator.device.do_download(UI.calculator.transferSize, data, false);
+      if (res==0){
+	alert(UI.langue==-1?'Erreur d\'ecriture flash interne. Tapez reset+6, installez le mode de recuperation et reessayez.':'Error writing internal flash. Type reset+6, run rescue mode and try again');
+	return 3;
+      }
+      UI.print((installboot?'Bootloader OK':'Khi internal OK')+', press RESET on calculator back');
+    }
+    else 
+      UI.print('Khi external OK');
     alert(UI.langue==-1?'Installation terminee. Appuyer sur RESET a l\'arriere de la calculatrice':'Install success. Press RESET on the calculator back.');
     return 0;
   },
@@ -316,9 +398,13 @@ var UI = {
     UI.calculator.device.startAddress = 0x20030000;
     let data=await UI.loadfile('recovery');
     await UI.calculator.device.clearStatus();
-    await UI.calculator.device.do_download(UI.calculator.transferSize, data, true);
+    let res=await UI.calculator.device.do_download(UI.calculator.transferSize, data, true);
+    if (res==0){
+      alert('Error sending recovery');
+      return 1;
+    }
     let tmp=confirm(UI.langue==-1?'La calculatrice devrait etre en mode recuperation. Cliquer sur le bouton Detecter puis sur install KhiCAS':'Calculator should be in rescue mode. Click on the Detect button then install KhiCAS.');
-    return;
+    return 0;
     if (tmp)
       UI.nws_detect(UI.numworks_install_delta,UI.nws_detect_failure);
     else
@@ -644,22 +730,25 @@ var UI = {
     UI.tar_writestring(buffer,"000000 ",offset+337,7); //devmajor
     UI.tar_writechecksum(buffer,offset);
   },
-  tar_adddata:function(buffer,filename,data,exec=0){
-    console.log('tar_adddata',exec);
+  tar_adddata:function(buffer,filename,data,exec=0,updatehtml=1){
+    console.log('tar_adddata',exec,UI.tar_first_modif_offset);
     let finfo=UI.tar_fileinfo(buffer,0);
-    let s=finfo.length;
-    if (s==0) return 0;
-    let last=finfo[s-1];
-    let offset=last.header_offset;
-    offset += UI.tar_filesize(last.size);
+    let s=finfo.length,offset=0;
+    if (s!=0){
+      let last=finfo[s-1];
+      offset=last.header_offset;
+      offset += UI.tar_filesize(last.size);
+    }
     let newsize=offset+1024+data.byteLength;
     newsize=10240*Math.ceil(newsize/10240);
     if (newsize>UI.numworks_maxtarsize) return 0;
     // console.log(buffer.byteLength,newsize);
     // resize buffer
-    if (buffer.byteLength<newsize)
-      buffer=UI.numworks_buffer=UI.arrayBufferTransfer(buffer,newsize);
-    // console.log(buffer.byteLength,newsize);
+    if (buffer.byteLength<newsize){
+      buffer=UI.arrayBufferTransfer(buffer,newsize);
+      if (updatehtml) UI.numworks_buffer=buffer;
+    }
+    console.log(buffer.byteLength,newsize);
     let view = new Uint8Array(buffer);    
     // add header
     // fill header with 0
@@ -675,8 +764,9 @@ var UI = {
     let srcview = new Uint8Array(data, 0, data.byteLength);    
     for (let i=0;i<data.byteLength;++i)
       view[offset+512+i]=srcview[i];
-    let F=UI.tar_fileinfo(buffer,1); //console.log(F);
-    return 1;
+    let F=UI.tar_fileinfo(buffer,updatehtml);
+    // console.log('tar_adddata',F);
+    return buffer;
   },
   tar_removefile:function(buffer,filename,really=1){
     let finfo=UI.tar_fileinfo(buffer,0);
@@ -715,6 +805,28 @@ var UI = {
     reader.readAsArrayBuffer(file);
     reader.onload = function(evt){
       let fname=file.name,exec=1;
+      if (fname.length>4 && fname.substr(fname.length-4,4)==".tar"){
+	let reader = new FileReader();
+	reader.onerror = function (evt) { }
+	reader.readAsArrayBuffer(file);
+	reader.onload = function(evt){
+	  let buffer=evt.target.result;
+	  let finfo=UI.tar_fileinfo(buffer,1),l=finfo.length;
+	  for (let i=0;i<l;++i){ 
+	    let cur=finfo[i];
+	    let name=cur.name;
+	    // check if name should be archived (not archived if it is in apps)
+	    let size=cur.size;
+	    let offset=cur.header_offset+512;
+	    let data=new Uint8Array(buffer,offset,size);
+	    if (UI.tar_adddata(UI.numworks_buffer,name,data,0,1)==0){
+	      alert(UI.langue==-1?'Pas d\'espace en memoire flash':'No space left in flash');
+	      return;
+	    }
+	  }
+	}
+	return;
+      }
       if (fname.length>4 && fname.substr(fname.length-4,4)==".zip"){
 	let zip=new JSZip();
 	zip.loadAsync(evt.target.result).then(
@@ -805,7 +917,7 @@ var UI = {
     }
   },
   numworks_gettar:function(calc){
-    if (calc==0){
+    if (calc<=0){
       UI.numworks_gettar_(calc);
       return;
     }
@@ -815,7 +927,7 @@ var UI = {
   },
   numworks_gettar_:async function(calc){
     console.log(calc,UI.calculator);
-    if (calc && (UI.calculator==0 || !UI.calculator_connected)){
+    if (calc>0 && (UI.calculator==0 || !UI.calculator_connected)){
       alert(UI.langue==-1?UI.chkmsgfr:chkmsgen);
       if (UI.calculator) UI.calculator.stopAutoConnect();
       return;
@@ -827,14 +939,76 @@ var UI = {
     UI.tar_first_modif_offset=0;
     if (calc==0)
       UI.numworks_buffer=await UI.loadfile('apps.tar');
+    else if (calc==-1)
+      UI.numworks_buffer=await UI.loadfile('appsalpha.tar');
     else
       UI.numworks_buffer=await UI.calculator.get_apps();
     let finfo=UI.tar_fileinfo(UI.numworks_buffer,1),s=finfo.length;
-    if (calc && s>0)
+    if (calc>0 && s>0)
       UI.tar_first_modif_offset=finfo[s-1].header_offset+UI.tar_filesize(finfo[s-1].size);
     // window.setTimeout(UI.tar_fileinfo,1000,UI.numworks_buffer);
     //UI.print("Archive flash:\n"+UI.numworks_tarinfo(UI.numworks_buffer));
   },
+  numworks_getarchive:function(filename,appendapps=1){
+    // filename=string filename to archive or 0 or 1 for calculator upgrade
+    UI.calc=2;
+    UI.nws_connect();
+    window.setTimeout(UI.numworks_getarchive_,100,filename,appendapps);
+  },
+  numworks_getarchive_:async function(filename,appendapps=1){
+    let sendcalc= !(typeof filename=="string");
+    if (sendcalc) appendapps=1;
+    console.log(sendcalc,UI.calculator);
+    if (UI.calculator==0 || !UI.calculator_connected){
+      alert(UI.langue==-1?UI.chkmsgfr:chkmsgen);
+      if (UI.calculator) UI.calculator.stopAutoConnect();
+      return;
+    }
+    let buffer=await UI.calculator.get_apps();
+    let appsbuffer;
+    if (filename==1)
+      appsbuffer=await UI.loadfile('appsalpha.tar');
+    else
+      appsbuffer=await UI.loadfile('apps.tar');
+    let appsfinfo=UI.tar_fileinfo(appsbuffer);
+    console.log('appsfinfo',appsfinfo);
+    let resbuffer;
+    if (appendapps)
+      resbuffer=appsbuffer;
+    else
+      resbuffer=new Uint8Array().buffer;
+    let finfo=UI.tar_fileinfo(buffer,0),l=finfo.length,L=appsfinfo.length;
+    for (let i=0;i<l;++i){ 
+      let cur=finfo[i];
+      let name=cur.name,j;
+      // check if name should be archived (not archived if it is in apps)
+      for (j=0;j<L;++j){
+	if (name==appsfinfo[j].name)
+	  break;
+      }
+      console.log(name,j,L);
+      if (j<L)
+	continue;
+      let size=cur.size;
+      let offset=cur.header_offset+512;
+      let data=new Uint8Array(buffer,offset,size);
+      //console.log(data);
+      resbuffer=UI.tar_adddata(resbuffer,name,data,0,0);
+      // console.log('getarchive',i,UI.tar_fileinfo(resbuffer,0));
+    }
+    finfo=UI.tar_fileinfo(resbuffer,0);
+    console.log(filename,finfo);
+    //return;
+    if (sendcalc){
+      UI.calculator.device.startAddress = 0x90200000;
+      UI.calculator.device.logProgress(0,'apps');
+      let res=await UI.calculator.device.do_download(UI.calculator.transferSize, resbuffer, false);
+      alert(UI.langue==-1?'Installation terminee. Appuyer sur RESET a l\'arriere de la calculatrice':'Install success. Press RESET on the calculator back.');
+      return res;
+    }
+    else
+      UI.file_savetar(resbuffer,filename);
+  },
   numworks_sendtar:function(){
     UI.calc=2;
     UI.nws_connect();
@@ -890,21 +1064,23 @@ var UI = {
 	return -1;
     //else alert(UI.langue==-1?'Le test va prendre environ 20 secondes':'Test will take about 20 seconds');
     UI.print('========');
-    let internal=await UI.calculator.get_internal_flash();
-    //console.log(sigfile);
-    let res=await UI.sig_check(sigfile,internal,'delta.internal.bin');
-    if (!res){
-      alert(UI.langue==-1?'Flash interne non certifiee. Cliquer sur installer KhiCAS pour installer un firmware certifie.':'Internal flash not certified');
-      return 1;
-    }
-    UI.print('Internal flash OK');
-    let external=await UI.calculator.get_external_flash();
+    let res;
+    let external=await UI.calculator.get_external_flash(1);
+    UI.adjust_exam_sector(external);
     res=await UI.sig_check(sigfile,external,'delta.external.bin');
     if (!res){
-      alert(UI.langue==-1?'Flash externe non certifiee. Cliquer sur installer KhiCAS pour installer un firmware certifie.':'External flash not certified');
+      alert(UI.langue==-1?'Flash externe slot 1 non certifiee. Cliquer sur installer KhiCAS pour installer un firmware certifie.':'External flash slot 1 not certified');
       return 2;
     }
-    UI.print('External flash OK');
+    UI.print('External flash slot 1 OK');
+    external=await UI.calculator.get_external_flash(2);
+    UI.adjust_exam_sector(external);
+    res=await UI.sig_check(sigfile,external,'khi.B.bin');
+    if (!res){
+      alert(UI.langue==-1?'Flash externe slot 2 non certifiee. Cliquer sur installer KhiCAS pour installer un firmware certifie.':'External flash slot 2 not certified');
+      return 2;
+    }
+    UI.print('External flash slot 2 OK');
     let apps=await UI.calculator.get_apps();
     res=await UI.sig_check(sigfile,apps,'apps.tar');
     if (!res){
@@ -913,11 +1089,13 @@ var UI = {
     }
     UI.print('Apps OK');
     if (rwcheck){
-      UI.print('R/W check 0x91200000');
-      res=await UI.calculator.rw_check(0x90120000,0xe0000);
-      if (!res){
-	alert(UI.langue==-1?'Echec du test lecture/ecriture':'Read/Write test failure');
-	return 4;
+      if (0){
+	UI.print('R/W check 0x91300000');
+	res=await UI.calculator.rw_check(0x90130000,0xd0000);
+	if (!res){
+	  alert(UI.langue==-1?'Echec du test lecture/ecriture':'Read/Write test failure');
+	  return 4;
+	}
       }
       UI.print('R/W check 0x90740000');
       res=await UI.calculator.rw_check(0x90740000,0xa0000);
@@ -927,7 +1105,20 @@ var UI = {
       }
       UI.print('R/W OK');
     }
-    alert(UI.langue==-1?'Firmware certifie':'Firmware certified');
+    let pinfo = await UI.calculator.getPlatformInfo();
+    if (pinfo["bootloader"]==0){
+      let internal=await UI.calculator.get_internal_flash();
+      //console.log(sigfile);
+      res=await UI.sig_check(sigfile,internal,'delta.internal.bin');
+      if (!res){
+	alert(UI.langue==-1?'Flash interne non certifiee. Cliquer sur installer KhiCAS pour installer un firmware certifie.':'Internal flash not certified');
+	return 1;
+      }
+      UI.print('Internal flash OK');
+      alert(UI.langue==-1?'Firmware certifie (interne, externe et KhiCAS)':'Certified firmware (internal, external and KhiCAS)');
+    }
+    else
+      alert(UI.langue==-1?'Firmware certifie (externe et KhiCAS). Pour certifier la flash interne, faire Reset et lancer KhiCAS.':'Firmware certified (external and KhiCAS). Reset the calculator and run KhiCAS to certify internal flash.');      
     return 0;
   },
   htmlbuffer:'',
