@@ -6,12 +6,8 @@
 #endif
 #include "vector.h"
 #include <string>
-#ifndef IN_GIAC
-#include <giac/giac.h>
-#else
 #include "giacPCH.h"
 #include "giac.h"
-#endif
 #ifdef HAVE_LIBFLTK
 #include <FL/Fl_Window.H>
 #include <FL/Fl_Menu.H>
@@ -24,11 +20,7 @@
 #ifdef HAVE_LC_MESSAGES
 #include <locale.h>
 #endif
-#ifndef IN_GIAC
-#include <giac/giacintl.h>
-#else
 #include "giacintl.h"
-#endif
 
 #ifndef NO_NAMESPACE_XCAS
 namespace xcas {
@@ -36,7 +28,7 @@ namespace xcas {
 
 #ifdef HAVE_LIBFLTK
 
-  Fl_Font cst_greek_translate(std::string & s);
+  Fl_Font cst_greek_translate(std::string & s,bool islabel=false);
   // utility for gen embedded widget memory desallocation
   void fltk_fl_widget_delete_function(void * ptr);
 
