@@ -42,11 +42,15 @@ namespace xcas {
   // Locales initialization, should be done before history.o is loaded
 
   std::string & xcas_locale();
+#ifndef USE_OBJET_BIDON // change by L. Marohnić
+  void localisation();
+#else  
   class objet_bidon{
   public:
     objet_bidon();
   };
-  // extern objet_bidon * mon_objet_bidon_ptr;
+  extern objet_bidon mon_objet_bidon;
+#endif
 
   void xcas_color(int color,bool dim3=false);
 
@@ -89,6 +93,7 @@ namespace xcas {
   void check_fl_rect(int i0,int j0,int i1,int j1,int imin,int jmin,int di,int dj,int delta_i,int delta_j);
   void check_fl_rectf(int i0,int j0,int i1,int j1,int imin,int jmin,int di,int dj,int delta_i,int delta_j);
   void check_fl_line(int i0,int j0,int i1,int j1,int imin,int jmin,int di,int dj,int delta_i,int delta_j);
+  void fl_line_color(int x1,int y1,int x2,int y2,int color);
 
   class Graph2d3d;
   extern Fl_Menu_Item Graph2d3d_menu[];
@@ -125,6 +130,7 @@ namespace xcas {
     giac::vecteur args_tmp; // WARNING should only contain numeric value
     unsigned args_tmp_push_size;
   public:
+    std::string modestr;
     std::vector<std::string> args_help;
     bool no_handle; // disable mouse handling
     bool show_mouse_on_object; // FL_MOVE always handled or not
@@ -150,7 +156,7 @@ namespace xcas {
     std::string title,x_axis_name,x_axis_unit,y_axis_name,y_axis_unit,z_axis_name,z_axis_unit,fcnfield,fcnvars;
     int npixels; // max # of pixels distance for click
     int show_axes,show_names;
-    giac::vecteur plot_instructions,animation_instructions,trace_instructions ;
+    giac::vecteur plot_instructions,animation_instructions,trace_instructions,native_renderer_instructions ;
     double animation_dt; // rate for animated plot
     bool paused;
     struct timeval animation_last; // clock value at last display
@@ -177,10 +183,17 @@ namespace xcas {
     int ntheta,nphi; // default discretization params for sphere drawing
     std::pair<Fl_Image *,Fl_Image *> * background_image; // 2-d only
     int x_axis_color,y_axis_color,z_axis_color;
+    int tracemode;
+    int tracemode_n; double tracemode_i; string tracemode_add; giac::vecteur tracemode_disp; double tracemode_mark;    
+    void init_tracemode();
+    void invert_tracemode();
+    void tracemode_set(int operation=0); // operation==1 if user is setting the value of t on a parametric curve, operation==2 for root, operation==3 for extremum, operation==4 mark current position, operation=5 for area
+    void draw_decorations();
     Graph2d3d(int x,int y,int w,int h,const char * l,double xmin,double xmax,double ymin,double ymax,double zmin,double zmax,double ortho,History_Pack * hp_);
     Graph2d3d(int x,int y,int w,int h,const char * l=0,History_Pack * hp_=0);
     double find_eps(); // find value of a small real wrt the current graph
     void update_infos(const giac::gen & g,const giac::context * contextptr);
+    giac::vecteur selected_names(bool allobjects,bool withdef) const;
     virtual void zoom(double d);
     virtual void zoomx(double d,bool round=false);
     virtual void zoomy(double d,bool round=false);
@@ -226,6 +239,13 @@ namespace xcas {
     void set_mode(const giac::gen & f_tmp,const giac::gen & f_final,int m);
     virtual void find_xyz(double i,double j,double depth,double & x,double & y,double & z);
     void find_title_plot(giac::gen & title_tmp,giac::gen & plot_tmp,const giac::context * contextptr);
+    int os_draw_string(int x_,int y_,int c,int bg,const char * s,bool fake=false) const;
+    int os_draw_string_small(int x_,int y_,int c,int bg,const char * s,bool fake=false) const ;
+    void os_set_pixel(int x,int y,int c) const;
+    void drawLine(int x1,int y1,int x2,int y2,int c) const;
+    void drawRectangle(int x_,int y_,int w_,int h_,int c) const ;
+    inline void os_fill_rect(int x_,int y_,int w_,int h_,int c) const {drawRectangle(x_,y_,w_,h_,c);}
+    virtual void update_rotation();
     void change_attributs();
     void change_attributs(int hp_pos);
     void resize_mouse_param_group(int W);
@@ -235,6 +255,13 @@ namespace xcas {
     std::string current_config();
     // return a sequence of graphic objects, ready for animation
     void adjust_cursor_point_type();
+    int top_margin() const;
+    int bottom_margin() const;
+    int right_margin() const;
+    int horz_pixels() const { return w()-right_margin(); }
+    int vert_pixels() const { return h()-top_margin()-bottom_margin(); }
+    double window_width() const { return window_xmax-window_xmin; }
+    double window_height() const { return window_ymax-window_ymin; }
   };
 
   class Graph2d:public Graph2d3d {
@@ -391,6 +418,26 @@ namespace xcas {
     virtual int handle(int event);
   };
 
+  typedef std::pair<Fl_Image *,Fl_Image *> img_ptr_pair_t;
+  class TextureCache {
+    std::map<std::string,img_ptr_pair_t*> _cache;
+    void _remove(img_ptr_pair_t *p,bool is_internal);
+public:
+    ~TextureCache();
+    img_ptr_pair_t *get(const std::string &s) const;
+    void insert(const std::string &s,std::pair<Fl_Image *,Fl_Image *> *iptr);
+    bool erase(const std::string &s);
+  };
+
+  class InternalImageCache {
+    std::map<std::string,Fl_Image *> _cache;
+public:
+    ~InternalImageCache();
+    Fl_Image * get(const std::string &s) const;
+    void insert(const std::string &s,Fl_Image * imgptr);
+    static bool is_internal_image_name(const std::string &s);
+  };
+
 #endif // HAVE_LIBFLTK
 
 #ifndef NO_NAMESPACE_XCAS
