@@ -19,11 +19,7 @@
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
-#ifndef IN_GIAC
-#include <giac/first.h>
-#else
 #include "first.h"
-#endif
 #include <string>
 #ifdef HAVE_LIBFLTK
 #include "Print.h"
@@ -905,7 +901,9 @@ namespace xcas {
       }
       ps_preview +="Program\\ Files/Ghostgum/gsview/gsview32.exe";
     }
+#ifndef __MINGW_H
     pngpdf |= 0x4;
+#endif
 #endif
     // translate into png, (inside GIMP Image->Transform->rotate, not anymore)
     // inside latex use includegraphics[width=\textwidth{filename}
