@@ -38,6 +38,10 @@ using namespace std;
 #include "lin.h"
 #include "intgab.h"
 #include "giacintl.h"
+#if defined GIAC_HAS_STO_38 || defined NSPIRE || defined NSPIRE_NEWLIB || defined FXCG || defined GIAC_GGB || defined USE_GMP_REPLACEMENTS || defined KHICAS
+#else
+#include "signalprocessing.h"
+#endif
 
 #ifndef NO_NAMESPACE_GIAC
 namespace giac {
@@ -84,7 +88,13 @@ namespace giac {
       }
       return gen(v,f0.subtype);
     }
-    gen t(s);
+    gen t(s),pt;
+#if defined GIAC_HAS_STO_38 || defined NSPIRE || defined NSPIRE_NEWLIB || defined FXCG || defined GIAC_GGB || defined USE_GMP_REPLACEMENTS || defined KHICAS
+#else
+    // addition by L.Marohnić: support for transforming periodic functions
+    if (laplace_periodic(f0,x,s,pt,contextptr))
+      return pt;
+#endif
     if (s==x){
 #ifdef GIAC_HAS_STO_38
       t=identificateur("s38_");
@@ -353,6 +363,13 @@ namespace giac {
       return gensizeerr(contextptr);
     if (has_num_coeff(f))
       return ilaplace(exact(f,contextptr),x,s,contextptr);
+#if defined GIAC_HAS_STO_38 || defined NSPIRE || defined NSPIRE_NEWLIB || defined FXCG || defined GIAC_GGB || defined USE_GMP_REPLACEMENTS || defined KHICAS
+#else
+    // addition by L.Marohnić: support for periodic summation
+    gen orig;
+    if (ilaplace2(f,x,s,orig,contextptr))
+      return orig;
+#endif
     gen remains,res=linear_apply(f,x,remains,0,contextptr,pf_ilaplace);
     res=subst(res,laplace_var,s,false,contextptr);
     if (!is_zero(remains))
