@@ -30,8 +30,31 @@
 #endif
 //#include <stdint.h>
 
-#if defined(__x86_64__) || defined(__arm64__)
+// mingw now defines x86_64
+#if (defined(__x86_64__) || defined(__arm64__)) && !defined __MINGW_H
 #define x86_64 1
+#else
+#ifdef __MINGW_H
+#define MINGW32
+#ifndef M_LN2
+#define M_LN2 0.693147180559945310
+#endif
+#ifndef M_PI
+#define M_PI       3.14159265358979323846
+#define M_PI_2       (M_PI/2)
+#endif
+#ifndef M_E
+#define M_E       2.71828182845904524
+#endif
+#ifndef M_SQRT2
+#define M_SQRT2       1.41421356237309505
+#endif
+#endif
+#endif
+
+#ifdef HP39
+#include <time.h>
+#define M_E 2.7182818284590452
 #endif
 
 // Thanks to Jason Papadopoulos, author of msieve
@@ -102,7 +125,7 @@ extern "C" void glcontext(int);
 #define CLOCK_T clock_t
 #else // EMCC
 #define CERR std::cerr
-#if defined(MS_SMART) || defined(NO_CLOCK)
+#if defined(MS_SMART) || defined(NO_CLOCK) || defined __MINGW_H
 #define CLOCK() 0
 #define CLOCK_T int
 #else
@@ -262,6 +285,7 @@ int my_sprintf(char * s, const char * format, ...);
 #endif
 
 #ifdef __VISUALC__ 
+#define _ITERATOR_DEBUG_LEVEL 0
 #define GIAC_GENERIC_CONSTANTS
 // Visual C++ is compiling
 #define VISUALC
@@ -313,7 +337,7 @@ inline void swap_giac_double(double & a,double & b){ double c=a; a=b; b=c; }
 #define swap_giac_double(a,b) std::swap<giac_double>(a,b)
 #endif
 
-#if defined WIN32 && defined x86_64
+#if defined x86_64
 typedef longlong ref_count_t;
 #else
 typedef int ref_count_t;
@@ -357,9 +381,16 @@ typedef int ref_count_t;
 #undef HAVE_GMPXX_H
 #undef HAVE_LIBMPFR
 #include "gmp_replacements.h"
-#else
+#else // USE_GMP_REPLACEMENTS
 #include <cstddef>
+#ifdef BF2GMP
+#include "bf2gmp.h"
+// #undef HAVE_LIBMPFR // to be replaced by defined later
+#undef HAVE_LIBMPFI // to be replaced by defined later
+#undef HAVE_GMPXX_H
+#else
 #include "gmp.h"
+#endif // BF2GMP
 #endif // USE_GMP_REPLACEMENTS
 
 #ifndef FXCG
@@ -519,14 +550,14 @@ inline int fsign (float f1){return f1==0?0:(f1>0?1:-1);}
 float fsqrt (float f1);
 void print_float(const giac_float & f,char * ch);
 inline float fpow(float f1,float f2){ 
-#ifdef NSPIRE
+#if defined NSPIRE || defined FXCG
   return pow(f1,f2); 
 #else
   return std::pow(f1,f2); 
 #endif
 }
 inline float ffloor(float f1){ 
-#ifdef NSPIRE
+#if defined NSPIRE || defined FXCG
   return floor(f1); 
 #else
   return std::floor(f1); 
@@ -545,7 +576,7 @@ inline float fgamma(float f1){ return gammaf(f1); } // or tgammaf(f1) on some ve
 #ifdef FXCG
 inline float atan2f(float f1,float f2,int rad){ if (rad) return std::atan2(f1,f2); else return std::atan2(f1,f2)*180/3.14159265358979323846;}
 #else
-inline float atan2f(float f1,float f2,int rad){ if (rad) return atan2f(f1,f2); else return atan2f(f1,f2)*180/M_PI;}
+inline float atan2f(float f1,float f2,int rad){ if (rad) return atan2f(f1,f2); else return atan2f(f1,f2)*180/3.14159265358979323846;}
 #endif
 #define fis_nan my_isnan
 #define fis_inf my_isinf
@@ -555,4 +586,31 @@ inline float atan2f(float f1,float f2,int rad){ if (rad) return atan2f(f1,f2); e
 #undef B0 //this conflicts with a define
 #undef bcopy //this conflicts with a define
 #endif
+
+#ifdef FXCG
+namespace ustl {
+  inline double abs(double d){ return ::fabs(d); }
+  inline double tan(double d){ return ::tan(d); }
+  inline double atan(double d){ return ::atan(d); }
+  inline double asin(double d){ return ::asin(d); }
+  inline double sin(double d){ return ::sin(d); }
+  inline double acos(double d){ return ::acos(d); }
+  inline double cos(double d){ return ::cos(d); }
+  inline double tanh(double d){ return ::tanh(d); }
+  inline double atanh(double d){ return ::atanh(d); }
+  inline double asinh(double d){ return ::asinh(d); }
+  inline double sinh(double d){ return ::sinh(d); }
+  inline double acosh(double d){ return ::acosh(d); }
+  inline double cosh(double d){ return ::cosh(d); }
+  inline double log(double d){ return ::log(d); }
+  inline double log10(double d){ return ::log10(d); }
+  inline double exp(double d){ return ::exp(d); }
+  inline double sqrt(double d){ return ::sqrt(d); }
+  inline double floor(double d){ return ::floor(d); }
+  inline double ceil(double d){ return ::ceil(d); }
+  inline double pow(double d1,double d2){ return ::pow(d1,d2); }
+  inline double atan2(double d1,double d2){ return ::atan2(d1,d2); }
+}
+#endif
+
 #endif // _GIAC_FIRST_H_
