@@ -1787,19 +1787,21 @@ menu = {
        	 { "normalt: Z-test",	function() menustring( "normalt(" ) end },
        	 { "studentt: Student-test",	function() menustring( "studentt(" ) end },
        },
-       { "Program",
-       	 { ";",	function() menustring( ";\n\r" ) end },
-       	 { "function",	function() menustring( "f(x):={ local y; y:=x*x; return y; }" ) end },
-       	 { "local",	function() menustring( "local ;\n" ) end },
-       	 { "return",	function() menustring( "return ;" ) end },
-       	 { "print",	function() menustring( "print(" ) end },
-       	 { "if/then",	function() menustring( "if then ; fi;" ) end },
-       	 { "if/then/else",	function() menustring( "if then ; else ; fi;" ) end },
-       	 { "for",	function() menustring( "for j from 1 to 5 do ; od;" ) end },
-       	 { "while",	function() menustring( "while do ; od;" ) end },
-       	 { "repeat",	function() menustring( "repeat ; until ;" ) end },
-       	 { "break",	function() menustring( "break;" ) end },
-       	 { "continue",	function() menustring( "continue;" ) end },
+       { "Plots",
+       	 { "plot(f(x),x=a..b)",	function() menustring( "plot(" ) end },
+       	 { "plotarea(f(x),x=a..b,[method])",	function() menustring( "plotarea(" ) end },
+       	 { "plotparam([x(t),y(t)],t=a..b)",	function() menustring( "plotparam(" ) end },
+       	 { "plotpolar([r(theta)],theta=a..b)",	function() menustring( "plotpolar(" ) end },
+       	 { "plotfield(f(t,y),[t=t1..t2,y=y1..y2])",	function() menustring( "plotfield(" ) end },
+       	 { "plotode(f(t,y),[t=t1..t2,y],[t0,y0])",	function() menustring( "plotode(" ) end },
+       	 { "plotdensity(f(x,y),[x=x1..x2,y=y1..y2])",	function() menustring( "plotdensity(" ) end },
+       	 { "plotcontour(f(x,y),[x=x1..x2,y=y1..y2])",	function() menustring( "plotcontour(" ) end },
+       	 { "plotimplicit(f(x,y),[x=x1..x2,y=y1..y2])",	function() menustring( "plotimplicit(" ) end },
+	 "-",
+       	 { "bar_plot(data_list)",	function() menustring( "bar_plot(" ) end },
+       	 { "histogram(data,xmin,size)",	function() menustring( "histogram(" ) end },
+       	 { "scatterplot(Xlist,Ylist)",	function() menustring( "scatterplot(" ) end },
+       	 { "linear_regression_plot(Xlist,Ylist)",	function() menustring( "linear_regression_plot(" ) end },
        },
        { "Lists",
        	 { "seq(expr,var,inf,sup)",	function() menustring( "seq(" ) end },
@@ -1817,7 +1819,10 @@ menu = {
        	 { "remove(function,l)",	function() menustring( "remove(" ) end },
        },
 	{
-		"Options",
+		"Shell Prog Options",
+		{ "open shell",	function() menustring( "*" ) end },
+		{ "open script editor",	function() menustring( "+\"\"" ) end },
+		"-",
 		{ "save variables",	function() menustring( "write(\"a.tns\",0" ) end },
 		{ "read variables",	function() menustring( "eval(read(\"a.tns\"))" ) end },
 		"-",
