@@ -27,6 +27,11 @@
 #include <sys/time.h>
 #endif
 
+#ifdef __MINGW_H
+inline int setenv(const char *name, const char *value, int overwrite){ return 0;}
+inline int unsetenv(const char *name){return 0;}
+#endif
+
 #ifndef NO_NAMESPACE_GIAC
 namespace giac {
 #endif // ndef NO_NAMESPACE_GIAC
@@ -172,6 +177,7 @@ namespace giac {
   gen _remove_language(const gen & args,GIAC_CONTEXT);
   gen _show_language(const gen & args,GIAC_CONTEXT);
   gen _os_version(const gen & args,GIAC_CONTEXT);
+  gen _linspace(const gen & args,GIAC_CONTEXT);
 
   extern const unary_function_ptr * const  at_normalize;
   void aplatir(const matrice & m,vecteur & v,bool full=false);
@@ -257,6 +263,7 @@ namespace giac {
 
   bool has_undef_stringerr(const gen & g,std::string & err);
   gen _is_polynomial(const gen & args,GIAC_CONTEXT);
+  bool is_periodic(const gen & f,const gen & x,gen & periode,GIAC_CONTEXT);
 
   // step by step utilities
   // poi=point of interest, tvi=table of variation
@@ -297,11 +304,21 @@ namespace giac {
   extern const unary_function_ptr * const  at_isinf ;
   extern const unary_function_ptr * const  at_isfinite ;
   extern const unary_function_ptr * const  at_isnan ;
+  extern const unary_function_ptr * const  at_fmod ;
   extern const unary_function_ptr * const  at_draw_string ;
   extern const unary_function_ptr * const  at_draw_filled_circle ;
   extern const unary_function_ptr * const  at_draw_filled_arc ;
+  extern const unary_function_ptr * const  at_draw_circle ;
+  extern const unary_function_ptr * const  at_draw_arc ;
+  extern const unary_function_ptr * const  at_draw_line ;
+  extern const unary_function_ptr * const  at_draw_polygon ;
+  extern const unary_function_ptr * const  at_fill_rect ;
+  extern const unary_function_ptr * const  at_draw_rectangle ;
   extern const unary_function_ptr * const  at_dtype ;
   extern const unary_function_ptr * const  at_rgb ;
+  extern const unary_function_ptr * const  at_hsv ;
+  extern const unary_function_ptr * const  at_hsv2rgb ;
+  extern const unary_function_ptr * const  at_rgb2hsv ;
   extern const unary_function_ptr * const  at_prediction;
   extern const unary_function_ptr * const  at_prediction95;
   extern const unary_function_ptr * const  at_confidence;
@@ -320,15 +337,25 @@ namespace giac {
   int rm(const char * filename);
 #endif
 
-  gen _show_pixels(const gen & args,GIAC_CONTEXT);
-  gen _rgb(const gen & args,GIAC_CONTEXT);
   gen _charpoly(const gen & args,GIAC_CONTEXT);
+  gen _show_pixels(const gen & args,GIAC_CONTEXT);
+  // additions by L.Marohnić
+  gen _rgb(const gen & args,GIAC_CONTEXT);
+  gen _hsv(const gen & args,GIAC_CONTEXT);
+  bool index2rgb(int c,unsigned char &r,unsigned char &g,unsigned char &b);
+  gen rgb2hsv(const vecteur &rgb,GIAC_CONTEXT);
+  gen hsv2rgb(const vecteur &hsv,GIAC_CONTEXT);
+  extern void (*fltk_colormap_rgb_ptr)(int c,unsigned char &r,unsigned char &g,unsigned char &b);
+  // end additions by LM
   extern bool freeze;
+  extern "C" void console_freeze();
+  extern "C" void sync_screen();
   void draw_rectangle(int x, int y, int width, int height, unsigned short color,GIAC_CONTEXT);
   void draw_polygon(std::vector< std::vector<int> > & v1,int color,GIAC_CONTEXT);
   void draw_filled_polygon(std::vector< std::vector<int> > &L,int xmin,int xmax,int ymin,int ymax,int color,GIAC_CONTEXT);
   void draw_line(int x1, int y1, int x2, int y2, int color,GIAC_CONTEXT);
   void set_pixel(int x,int y,int c,GIAC_CONTEXT);
+  extern "C" void set_pixel(int x,int y,int c);
   void draw_circle(int xc,int yc,int r,int color,bool q1,bool q2,bool q3,bool q4,GIAC_CONTEXT);
   void draw_filled_circle(int xc,int yc,int r,int color,bool left,bool right,GIAC_CONTEXT);
   void draw_arc(int xc,int yc,int rx,int ry,int color,double t1, double t2,bool q1,bool q2,bool q3,bool q4,GIAC_CONTEXT);
