@@ -781,7 +781,8 @@ namespace giac {
       res.push_back(vector_giac_double_2_vecteur(v[i]));
     return res;
   }  
-  
+
+#if 0  // moved to signalprocessing.cc
   gen _hilbert(const gen & args,GIAC_CONTEXT){
     if ( args.type==_STRNG && args.subtype==-1) return  args;
     int n,p;
@@ -811,6 +812,7 @@ namespace giac {
   static const char _hilbert_s[]="hilbert";
   static define_unary_function_eval (__hilbert,&_hilbert,_hilbert_s);
   define_unary_function_ptr5( at_hilbert ,alias_at_hilbert,&__hilbert,0,true);
+#endif
 
   gen l2norm2(const gen & g){
     if (g.type!=_VECT)
@@ -1427,7 +1429,7 @@ namespace giac {
     if ( args.type==_STRNG && args.subtype==-1) return  args;
     int n;
     gen a,x;
-    if (args.type==_VECT && args._VECTptr->size()==2 && is_integer(args._VECTptr->front()))
+    if (args.type==_VECT && args._VECTptr->size()==2 && is_integer(args._VECTptr->front()) && args._VECTptr->back().type!=_IDNT)
       return tchebyshev_eval(args._VECTptr->front(),args._VECTptr->back(),makevecteur(1,args._VECTptr->back()),contextptr);
     if (!find_n_x(args,n,x,a))
       return gensizeerr(contextptr);
@@ -1452,8 +1454,8 @@ namespace giac {
   }
   gen _tchebyshev2(const gen & args,GIAC_CONTEXT){
     if ( args.type==_STRNG && args.subtype==-1) return  args;
-    if (args.type==_VECT && args._VECTptr->size()==2 && is_integer(args._VECTptr->front()))
-      return tchebyshev_eval(args._VECTptr->front(),args._VECTptr->back(),makevecteur(0,1),contextptr);
+    if (args.type==_VECT && args._VECTptr->size()==2 && is_integer(args._VECTptr->front()) && args._VECTptr->back().type!=_IDNT)
+      return tchebyshev_eval(args._VECTptr->front()+1,args._VECTptr->back(),makevecteur(0,1),contextptr);
     int n;
     gen a,x;
     if (!find_n_x(args,n,x,a))
