@@ -491,6 +491,7 @@ namespace giac {
     log_output_redirect(GIAC_CONTEXT) { old=logptr(ctx=contextptr)->rdbuf(buffer.rdbuf()); }
     ~log_output_redirect() { logptr(ctx)->rdbuf(old); }
     std::string get_buffer_string() const { return buffer.str(); }
+    bool has_warning() const;
   private:
     const context *ctx;
     std::stringstream buffer;
@@ -500,7 +501,7 @@ namespace giac {
   // LDL decomposition, inertia computation and solve_indef for fast system solving using the factorization
   bool ldl(matrice & a,std::vector<int> & perm,int mat_type,bool &sing,double time_limit,GIAC_CONTEXT);
 #ifdef HAVE_LIBLAPACK
-  bool solve_indef(double *A,double *WORK,int *IPIV,double *b,int N,int NRHS,int *p,int *n,int *z,GIAC_CONTEXT);
+  bool solve_indef(double *A,double **WORK,int *IPIV,double *b,int N,int NRHS,int *p,int *n,int *z);
 #endif
   bool solve_indef(matrice &A,const vecteur *b,vecteur &x,int *p,int *n,int *z,GIAC_CONTEXT);
   gen _ldl(const gen & a,GIAC_CONTEXT);
